package ctbrec.sites.streamray;

import static ctbrec.io.HttpConstants.*;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.json.JSONArray;
import org.json.JSONObject;
import okhttp3.Request;
import okhttp3.Response;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ctbrec.Model;
import ctbrec.StringUtil;
import ctbrec.io.HttpClient;
import ctbrec.io.HttpException;
import ctbrec.sites.AbstractSite;


public class Streamray extends AbstractSite {

    private static final Logger LOG = LoggerFactory.getLogger(Streamray.class);
    
    private StreamrayHttpClient httpClient;
    public static String domain = "streamray.com";
    public static String baseUri = "https://streamray.com";
    public static String apiURL = "https://beta-api.cams.com";

    @Override
    public void init() throws IOException {}

    @Override
    public String getName() {
        return "Streamray";
    }

    @Override
    public String getBaseUrl() {
        return baseUri;
    }

    public String getApiUrl() {
        return apiURL;
    }

    @Override
    public StreamrayModel createModel(String name) {
        StreamrayModel model = new StreamrayModel();
        model.setName(name);
        model.setUrl(getBaseUrl() + '/' + name);
        model.setDescription("");
        model.setSite(this);
        return model;
    }

    @Override
    public Double getTokenBalance() throws IOException {
        return 0d;
    }

    @Override
    public String getBuyTokensLink() {
        return getBaseUrl();
    }

    @Override
    public synchronized boolean login() throws IOException {
        boolean result = getHttpClient().login();
        LOG.debug("Streamray site login call result: {}", result);
        return result;
    }

    @Override
    public HttpClient getHttpClient() {
        if (httpClient == null) {
            httpClient = new StreamrayHttpClient(getConfig());
        }
        return httpClient;
    }

    @Override
    public void shutdown() {
        if (httpClient != null) {
            httpClient.shutdown();
        }
    }

    @Override
    public boolean supportsTips() {
        return false;
    }

    @Override
    public boolean supportsFollow() {
        return false;
    }

    @Override
    public boolean supportsSearch() {
        return true;
    }

    @Override
    public boolean searchRequiresLogin() {
        return false;
    }

    @Override
    public List<Model> search(String q) throws IOException, InterruptedException {
        if (StringUtil.isBlank(q)) {
            return Collections.emptyList();
        }
        String url = getApiUrl() + "/models/new/?limit=30&search=" + URLEncoder.encode(q, "utf-8") + "&order=is_online";
        Request req = new Request.Builder()
                .url(url)
                .header(USER_AGENT, getConfig().getSettings().httpUserAgent)
                .build();
        try (Response response = getHttpClient().execute(req)) {
            if (response.isSuccessful()) {
                JSONObject json = new JSONObject(response.body().string());
                if (json.has("results")) {
                    List<Model> models = new ArrayList<>();
                    JSONArray results = json.getJSONArray("results");
                    if (results.length() == 0) {
                        return Collections.emptyList();    
                    }
                    for (int i = 0; i < results.length(); i++) {
                        JSONObject result = results.getJSONObject(i);
                        StreamrayModel model = createModel(result.getString("stream_name"));
                        String image = result.optString("profile_image");
                        if (StringUtil.isBlank(image)) {
                            image = model.getPreviewURL();
                        }
                        model.setPreview(image);
                        models.add(model);
                    }
                    return models;
                } else {
                    return Collections.emptyList();
                }
            } else {
                throw new HttpException(response.code(), response.message());
            }
        }
    }

    @Override
    public boolean isSiteForModel(Model m) {
        return m instanceof StreamrayModel;
    }

    @Override
    public boolean credentialsAvailable() {
        return true;
    }

    @Override
    public Model createModelFromUrl(String url) {
        Matcher m = Pattern.compile("https://(streamray|cams).com/([_a-zA-Z0-9]+)").matcher(url);
        if (m.matches()) {
            String modelName = m.group(2);
            return createModel(modelName);
        } else {
            return super.createModelFromUrl(url);
        }
    }

    @Override
    public String getAffiliateLink() {
        return getBaseUrl();
    }
}
